@@ -104,10 +104,11 @@ pub struct DeepEx<'a, T: Copy + Debug> {
 }
 
 fn lift_nodes<'a, T: Copy + Debug>(deepex: &mut DeepEx<'a, T>) {
+
     if deepex.nodes.len() == 1 && deepex.unary_op.op.len() == 0 {
-        match &deepex.nodes[0].clone() {
+        match deepex.nodes[0].clone() {
             DeepNode::Expr(e) => {
-                *deepex = e.clone();
+                *deepex = e;
             }
             _ => (),
         }
@@ -696,9 +697,9 @@ fn test_partial_finite() {
         }
     }
 
-    test("z*sin(x)+cos(y)^(1 + x^2)/(sin(z))", &ops, 0.001..1.0);
+    test("z*sin(x)+cos(y)^(1 + x^2)/(sin(z))", &ops, 0.01..1.0);
     test("sin(sin(x+z))", &ops, -10.0..10.0);
-    test("x^x", &ops, 0.001..2.0);
+    test("x^x", &ops, 0.01..2.0);
     test("x^y", &ops, 4.036286084344371..4.036286084344372);
     test("z+sin(x)+cos(y)", &ops, -1.0..1.0);
     test("sin(cos(sin(z)))", &ops, -10.0..10.0);
