@@ -83,6 +83,94 @@ pub fn parsed_tokens_to_deepex<'a, T: Copy + FromStr + Debug>(
     Ok(expr)
 }
 
+fn is_operator_binary<T: Copy + FromStr>(
+    op: &Operator<T>,
+    parsed_token_on_the_left: &ParsedToken<T>,
+) -> bool {
+    match op.unary_op {
+        None => true,
+        Some(_) => match parsed_token_on_the_left {
+            ParsedToken::Num(_) | ParsedToken::Var(_) | ParsedToken::Paren(_) => true,
+            ParsedToken::Op(_) => false,
+        },
+    }
+}
+
+fn find_var_index<'a>(name: &str, parsed_vars: &[&'a str]) -> usize {
+    let idx = parsed_vars.iter().enumerate().find(|(_, n)| **n == name);
+    match idx {
+        Some((i, _)) => i,
+        None => {
+            panic!("This is probably a bug. I don't know variable {}", name)
+        }
+    }
+}
+
+/// Handles the case that a token is a unary operator and returns a tuple.
+/// The first element is a node that is either an expression with a unary operator or a
+/// number where the unary operator has been applied to. the second element is the number
+/// of tokens that are covered by the unary operator and its argument. Note that a unary
+/// operator can be a composition of multiple functions.
+fn process_unary<'a, T: Copy + FromStr + Debug>(
+    token_idx: usize,
+    unary_op: fn(T) -> T,
+    repr: &'a str,
+    parsed_tokens: &[ParsedToken<'a, T>],
+    parsed_vars: &[&'a str],
+) -> Result<(DeepNode<'a, T>, usize), ExParseError> {
+    // gather subsequent unary operators from the beginning
+    let iter_of_uops = iter::once((repr, unary_op)).chain(
+        (token_idx + 1..parsed_tokens.len())
+            .map(|j| match parsed_tokens[j] {
+                ParsedToken::Op(op) => (op.repr, op.unary_op),
+                _ => ("", None),
+            })
+            .take_while(|(_, uo)| uo.is_some())
+            .map(|(repr_, uo)| (repr_, uo.unwrap())),
+    );
+    let vec_of_uops = iter_of_uops
+        .clone()
+        .map(|(_, uo_)| uo_)
+        .collect::<VecOfUnaryFuncs<_>>();
+    let vec_of_uop_reprs = iter_of_uops
+        .clone()
+        .map(|(repr_, _)| repr_)
+        .collect::<Vec<_>>();
+    let n_uops = vec_of_uops.len();
+    let uop = UnaryOp::from_vec(vec_of_uops);
+    match &parsed_tokens[token_idx + n_uops] {
+        ParsedToken::Paren(_) => {
+            let (expr, i_forward) = make_expression::<T>(
+                &parsed_tokens[token_idx + n_uops + 1..],
+                parsed_vars,
+                UnaryOpWithReprs {
+                    reprs: vec_of_uop_reprs,
+                    op: uop,
+                },
+            )?;
+            Ok((DeepNode::Expr(expr), i_forward + n_uops + 1))
+        }
+        ParsedToken::Var(name) => {
+            let expr = DeepEx::new(
+                vec![DeepNode::Var((find_var_index(name, &parsed_vars), name))],
+                BinOpsWithReprs {
+                    reprs: Vec::new(),
+                    ops: BinOpVec::new(),
+                },
+                UnaryOpWithReprs {
+                    reprs: vec_of_uop_reprs,
+                    op: uop,
+                },
+            )?;
+            Ok((DeepNode::Expr(expr), n_uops + 1))
+        }
+        ParsedToken::Num(n) => Ok((DeepNode::Num(uop.apply(*n)), n_uops + 1)),
+        _ => Err(ExParseError {
+            msg: "Invalid parsed token configuration".to_string(),
+        }),
+    }
+}
+
 /// Returns an expression that is created recursively and can be evaluated
 ///
 /// # Arguments
@@ -103,144 +191,41 @@ pub fn make_expression<'a, T>(
 where
     T: Copy + FromStr + Debug,
 {
-    fn unpack_binop<S>(bo: Option<BinOp<S>>) -> BinOp<S>
-    where
-        S: Copy + FromStr + Debug,
-    {
-        match bo {
-            Some(bo) => bo,
-            None => panic!("This is probably a bug. Expected binary operator but there was none."),
-        }
-    }
-
-    let find_var_index = |name: &str| {
-        let idx = parsed_vars.iter().enumerate().find(|(_, n)| **n == name);
-        match idx {
-            Some((i, _)) => i,
-            None => {
-                panic!("This is probably a bug. I don't know variable {}", name)
-            }
-        }
-    };
-    // this closure handles the case that a token is a unary operator and accesses the
-    // variable 'tokens' from the outer scope
-    let process_unary = |i: usize, uo, repr| {
-        // gather subsequent unary operators from the beginning
-        let iter_of_uops = iter::once((repr, uo)).chain(
-            (i + 1..parsed_tokens.len())
-                .map(|j| match parsed_tokens[j] {
-                    ParsedToken::Op(op) => (op.repr, op.unary_op),
-                    _ => ("", None),
-                })
-                .take_while(|(_, uo_)| uo_.is_some())
-                .map(|(repr_, uo_)| (repr_, uo_.unwrap())),
-        );
-        let vec_of_uops = iter_of_uops
-            .clone()
-            .map(|(_, uo_)| uo_)
-            .collect::<VecOfUnaryFuncs<_>>();
-        let vec_of_uop_reprs = iter_of_uops
-            .clone()
-            .map(|(repr_, _)| repr_)
-            .collect::<Vec<_>>();
-        let n_uops = vec_of_uops.len();
-        let uop = UnaryOp::from_vec(vec_of_uops);
-        match &parsed_tokens[i + n_uops] {
-            ParsedToken::Paren(p) => match p {
-                Paren::Close => Err(ExParseError {
-                    msg: "closing parenthesis after an operator".to_string(),
-                }),
-                Paren::Open => {
-                    let (expr, i_forward) = make_expression::<T>(
-                        &parsed_tokens[i + n_uops + 1..],
-                        parsed_vars,
-                        UnaryOpWithReprs {
-                            reprs: vec_of_uop_reprs,
-                            op: uop,
-                        },
-                    )?;
-                    Ok((DeepNode::Expr(expr), i_forward + n_uops + 1))
-                }
-            },
-            ParsedToken::Var(name) => {
-                let expr = DeepEx::new(
-                    vec![DeepNode::Var((find_var_index(name), name))],
-                    BinOpsWithReprs {
-                        reprs: Vec::new(),
-                        ops: BinOpVec::new(),
-                    },
-                    UnaryOpWithReprs {
-                        reprs: vec_of_uop_reprs,
-                        op: uop,
-                    },
-                )?;
-                Ok((DeepNode::Expr(expr), n_uops + 1))
-            }
-            ParsedToken::Num(n) => Ok((DeepNode::Num(uop.apply(*n)), n_uops + 1)),
-            ParsedToken::Op(_) => Err(ExParseError {
-                msg: "a unary operator cannot be followed by a binary operator".to_string(),
-            }),
-        }
-    };
-
     let mut bin_ops = BinOpVec::new();
     let mut reprs_bin_ops: Vec<&str> = Vec::new();
     let mut nodes = Vec::<DeepNode<T>>::new();
-
+    let make_both_ops_none_error = |op: &Operator<T>| ExParseError {
+        msg: format!("operator {} is neither unary nor binary", op.repr),
+    };
     // The main loop checks one token after the next whereby sub-expressions are
     // handled recursively. Thereby, the token-position-index idx_tkn is increased
     // according to the length of the sub-expression.
     let mut idx_tkn: usize = 0;
     while idx_tkn < parsed_tokens.len() {
         match &parsed_tokens[idx_tkn] {
-            ParsedToken::Op(op) => match op.unary_op {
-                None => {
-                    bin_ops.push(unpack_binop(op.bin_op));
+            ParsedToken::Op(op) => {
+                if idx_tkn > 0 && is_operator_binary(&op, &parsed_tokens[idx_tkn - 1]) {
+                    bin_ops.push(op.bin_op.ok_or(make_both_ops_none_error(op))?);
                     reprs_bin_ops.push(op.repr);
                     idx_tkn += 1;
+                } else {
+                    let (node, idx_forward) = process_unary(
+                        idx_tkn,
+                        op.unary_op.ok_or(make_both_ops_none_error(op))?,
+                        op.repr,
+                        &parsed_tokens,
+                        &parsed_vars,
+                    )?;
+                    nodes.push(node);
+                    idx_tkn += idx_forward;
                 }
-                Some(uo) => {
-                    // might the operator be unary?
-                    if idx_tkn == 0 {
-                        // if the first element is an operator it must be unary
-                        let (node, idx_forward) = process_unary(idx_tkn, uo, op.repr)?;
-                        nodes.push(node);
-                        idx_tkn += idx_forward;
-                    } else {
-                        // decide type of operator based on predecessor
-                        match &parsed_tokens[idx_tkn - 1] {
-                            ParsedToken::Num(_) | ParsedToken::Var(_) => {
-                                // number or variable as predecessor means binary operator
-                                bin_ops.push(unpack_binop(op.bin_op));
-                                reprs_bin_ops.push(op.repr);
-                                idx_tkn += 1;
-                            }
-                            ParsedToken::Paren(p) => match p {
-                                Paren::Open => {
-                                    let msg = "This is probably a bug. An opening paren cannot be the predecessor of a binary operator.";
-                                    panic!("{}", msg);
-                                }
-                                Paren::Close => {
-                                    bin_ops.push(unpack_binop(op.bin_op));
-                                    reprs_bin_ops.push(op.repr);
-                                    idx_tkn += 1;
-                                }
-                            },
-                            ParsedToken::Op(_) => {
-                                let (node, idx_forward) = process_unary(idx_tkn, uo, op.repr)?;
-                                nodes.push(node);
-                                idx_tkn += idx_forward;
-                            }
-                        }
-                    }
-                }
-            },
+            }
             ParsedToken::Num(n) => {
                 nodes.push(DeepNode::Num(*n));
                 idx_tkn += 1;
             }
             ParsedToken::Var(name) => {
-                nodes.push(DeepNode::Var((find_var_index(name), name)));
+                nodes.push(DeepNode::Var((find_var_index(name, &parsed_vars), name)));
                 idx_tkn += 1;
             }
             ParsedToken::Paren(p) => match p {
