@@ -2,3 +2,5 @@ pub mod deep;
 mod deep_details;
 pub mod flat;
 mod partial_derivatives;
+#[cfg(feature = "serde_support")]
+mod serde;
\ No newline at end of file
