@@ -1,4 +1,4 @@
-#![doc(html_root_url = "https://docs.rs/exmex/0.9.1")]
+#![doc(html_root_url = "https://docs.rs/exmex/0.9.2")]
 //! Exmex is a fast, simple, and **ex**tendable **m**athematical **ex**pression evaluator
 //! with the ability to compute [partial derivatives](FlatEx::partial) of expressions.  
 //! ```rust
@@ -183,6 +183,12 @@
 //! # }
 //! ```
 //!
+//! ## Serialization and Deserialization
+//!
+//! To use [`serde`](https://serde.rs/) you can activate the feature `serde_support`. 
+//! Currently, this only works for default operators. The implementation 
+//! un-parses and re-parses the whole expression.
+//!
 //! ## Unicode
 //! Unicode input strings are currently not supported 😕 but might be added in the
 //! future 😀.
@@ -529,11 +535,19 @@ mod tests {
                 bin_op: None,
                 unary_op: Some(|a: f32| 1.0 / a),
             },
+            Operator {
+                repr: "None",
+                bin_op: None,
+                unary_op: None,
+            }
         ];
         let expr = parse("2**2*invert(3)", &custom_ops).unwrap();
         let val = expr.eval(&[]).unwrap();
         assert_float_eq_f32(val, 4.0 / 3.0);
 
+        let expr = parse("None(5)", &custom_ops);
+        assert!(expr.is_err());
+
         let zero_mapper = Operator {
             repr: "zer0",
             bin_op: Some(BinOp {
@@ -550,6 +564,7 @@ mod tests {
         let expr = parse("2^2*1/(berti) + zer0(4)", &extended_operators).unwrap();
         let val = expr.eval(&[4.0]).unwrap();
         assert_float_eq_f32(val, 1.0);
+
     }
 
     #[test]
@@ -664,7 +679,7 @@ mod tests {
         let flatex = parse_with_default_ops::<f64>(sut).unwrap();
         let deri = flatex.partial(var_idx).unwrap();
         let reference = |x: f64| 1.0 / x * x.exp() + x.ln() * x.exp();
-        test(var_idx, n_vars, 0.001..100.0, &deri, reference);
+        test(var_idx, n_vars, 0.01..100.0, &deri, reference);
 
         let sut = "a+z+sinh(v)/cosh(v)+b+tanh({v})";
         println!("{}", sut);
@@ -787,4 +802,13 @@ mod tests {
         assert!(eval_str(")2*(5+5)*3-2)*2").is_err());
         assert!(eval_str("2*(5+5))").is_err());
     }
+
+    #[cfg(feature="serde_support")]
+    #[test]
+    fn test_serde_public_interface() {
+        let s = "{x}^(3.0-{y})";
+        let flatex = parse_with_default_ops::<f64>(s).unwrap();
+        let serialized = serde_json::to_string(&flatex).unwrap();
+        let deserialized = serde_json::from_str::<FlatEx<f64>>(serialized.as_str()).unwrap();
+        assert_eq!(s, format!("{}", deserialized));    }
 }
